
AppTitle "On a roll - Demonstration"

Graphics 800,600 , 0,2  ; larger window size
Const sw=640,sh=480
Graphics sw,sh , 0,3  ; smaller (scaled) graphics content
SetBuffer BackBuffer()
Global r1=192,g1=224,b1=255  ; inner text color (body)
Global r2=0,g2=0,b2=0  ; outer text color (border)
Const r3=0,g3=64,b3=128  ; background color
ClsColor r3,g3,b3
Cls
SetFont LoadFont( "Lucida Console" , 20 )
TextOutLine sw/2,sh/2 , "LOADING" , True,True , 0,0,0 , r3/2,g3/2,b3/2
Flip False

Include "INC\input.def.bb"
Include "INC\render.lib.bb"
Include "INC\timersystem 4c.bb"
Include "INC\string.lib.bb"
Include "INC\math.lib.bb"

Include "INC\bassmod.bb"
If Not BASSMOD_Init( -1 , 44100 , 0 )
    RuntimeError "Unable to initialize BASSMOD:"+Chr(13)+BASSMOD_GetErrorString$()
EndIf

Const spec$ = "MUS\ONAROLL.IT"
If FileType(spec) <> 1
    RuntimeError "Unable to find music file"
End If
; loop the music endlessly, and smoothen sharp sample cuts
Const flags = BASS_MUSIC_LOOP + BASS_MUSIC_RAMPS
Local music = BASSMOD_MusicLoad( 0 , spec , 0,0 , flags )
If Not music
    RuntimeError "Unable to load music file:"+Chr(13)+BASSMOD_GetErrorString$()
End If

; perhaps use only the amplification for independently controlling the music volume
; although, depending on the music, high values (85+) can cause distortion
Local amp = 80  ; default = 50 (min=0,max=100)
BASSMOD_MusicSetAmplify amp

; bassmod's set/get volume seems to mess with Windows' "wave" volume
; which overrides a user's preference (!) and leaves it altered after quitting (!)
Local vol = BASSMOD_GetVolume()  ; for Windows i usually recommended a default of 100
BASSMOD_SetVolume vol

Local pansep = 50  ; default = 50 (min=0,max=100)
BASSMOD_MusicSetPanSep pansep

BASSMOD_MusicPlay

MilliSecs = MilliSecs()
Cls
Flip

; wait for the player to begin (perfect sync!)
Local timeout_timer = NewTimer( 3333 )
Repeat
    Delay 5
    MilliSecs = MilliSecs()
    If TimerDone( timeout_timer )
        RuntimeError "Unable to start the music"
    End If
Until BASSMOD_MusicIsActive() = BASS_ACTIVE_PLAYING

Const check_oldrow = 63
Const check_currow = 0
Local oldrow = check_oldrow
Local currow = check_currow
Local ordstart = MilliSecs
Local ordtime = 0

Local musictimer = NewTimer( infinity )  ; keep track of music time
; schism tracker: 10846
Const imagetime = 10889  ; how long to show each image  (sync'ed to music speed)
Const transtime =   250  ; how long the transition takes (between images)
Local imagetimer = NewTimer( imagetime )  ; look at img for a while (sync'ed to ordernum)
; move ahead in time for the transition to end in perfect sync with the next music order
ShiftTimer imagetimer , transtime

Local imgsel = 0  ; currently selected image (for display)
Local imgnew = 1  ; which (new) image to transition to
Const imgfiles = 5
Local img[imgfiles]

.imagefiles
Data "yatoo shuttle.jpg"
Data "Cloud City.jpg"
Data "F-19 Stealth.jpg"
Data "formation.jpg"
Data "f-22 raptor.JPG"

Restore imagefiles
Const imgdir$ = "GFX\"
Local imgnum,imgfile$
For imgnum = 1 To imgfiles
    Read imgfile
    img[imgnum] = LoadImage( imgdir+imgfile )
    If Not img[imgnum]
        RuntimeError "Unable to load image file:"+Chr(13)+imgfile
    EndIf
Next

StartTransition img[imgnew] , img[imgsel] , transtime*2
Local first_transition = True

Local textimage = CreateImage( sw,sh )
Local texttimer = NewTimer( 66 )  ; update text on interval
Local hide_text = True
Global detail_text = True
Local vsync = True
Local debug_text = False

Local fpstimer = NewTimer( 1000 )
Local fps,frames

Local dial_angle
Local dial_width = 16
Local dial_height = 12

Const sx=20,sy=16  ; spacing per char
Const tx=sx/1.5,ty=sy/1.5  ; text position
Repeat
    MilliSecs = MilliSecs()
    
    ; debug: track order duration using rows
    ; - perhaps best way is to sync image transition to rows instead of time
    If music_row() <> currow
        oldrow = currow
        currow = music_row()
        If currow = check_currow And oldrow = check_oldrow
            ordtime = MilliSecs - ordstart
            ordstart = MilliSecs
        End If
    End If
    
    ; toggles and debug
    If KeyHit(53)  ; slash ('/')
        vsync = Not vsync
    End If
    If KeyHit(43)  ; backslash ('\')
        detail_text = Not detail_text
    End If
    If KeyHit(59)  ; F1
        hide_text = Not hide_text
    End If
    If KeyHit(88)  ; F12
        debug_text = Not debug_text
    End If
    If KeyHit(40)  ; quote (')
        r1 = 255 - r1
        g1 = 255 - g1
        b1 = 255 - b1
        r2 = 255 - r2
        g2 = 255 - g2
        b2 = 255 - b2
    End If
    
    ; music control
    If KeyDown(key_up) Or KeyDown(key_down)
        vol = vol + 1 * ( KeyDown(key_up) - KeyDown(key_down) )
        If vol > 100 Then vol = 100
        If vol < 0 Then vol = 0
        BASSMOD_SetVolume vol
    End If
    If KeyDown(key_left) Or KeyDown(key_right)
        amp = amp + 1 * ( KeyDown(key_right) - KeyDown(key_left) )
        If amp > 100 Then amp = 100
        If amp < 0 Then amp = 0
        BASSMOD_MusicSetAmplify amp
    End If
    If KeyHit(key_space)
        Select BASSMOD_MusicIsActive()
            Case BASS_ACTIVE_PLAYING
                BASSMOD_MusicPause
                PauseTimer musictimer
                PauseTimer imagetimer
            Case BASS_ACTIVE_PAUSED
                BASSMOD_MusicPlay
                ResumeTimer musictimer
                ResumeTimer imagetimer
        End Select
    End If
    
    ; image transitions
    If KeyHit(key_enter)
        Local skipimg = True
        ; prevent timer from elapsing this round
        If TimerRemain( imagetimer ) <= TimerInterval( imagetimer )
            ; give more time  (add the interval to keep sync)
            ShiftTimer imagetimer , -TimerInterval( imagetimer )
        End If
    End If
    If TimerDone( imagetimer ) Or skipimg
        skipimg = False
        imgnew = imgsel + 1
        If imgnew > imgfiles
            imgnew = 1
        End If
        StartTransition img[imgnew] , img[imgsel] , transtime
    End If
    If UpdateTransition()
        imgsel = imgnew
        imgnew = 0
        first_transition = False
    End If
    If Not TransitionActive()
        DrawBlock img[imgsel] , 0,0
    End If
    
    ; frames per second
    frames = frames + 1
    If TimerDone( fpstimer )
        fps = frames
        frames = 0
    End If
    
    ; text display
    If Not first_transition
        If TimerDone( texttimer )
            SetBuffer ImageBuffer( textimage )
            Color 0,0,0
            Rect 0,0 , sw,sh , True
            If hide_text
                textoutR 0 , "<F1>"
            Else
                textoutR 0 , "Help   <F1>"
                textout 0 , "<Space>      Playing:  "+BASSMOD_MusicIsActive()
                textout 1 , "<Up/Down>    Volume:   "+BASSMOD_GetVolume()+"%"
                textout 2 , "<Left/Right> Amplify:  "+amp+"%"
                textout 3 , "             Position:  Order "+music_order()+"  Row "+music_row()
                textout 4 , "             Duration: "+String_FormatTime( TimerElapsed(musictimer) )
                textoutR 1 , "Debug  <F12>"
                If debug_text
                    textout 6 , "<Enter>      Next picture: "+String_FormatTime( TimerRemain(imagetimer) )
                    textout 7 , "             Order duration: "+String_WithCommas( ordtime )+" ms"
                    textout 9 , "</>          Vertical sync: "+String_BooleanText( vsync , "On" )
                    textout 10, "<\>          Text border: "+String_BooleanText( detail_text , "On" )
                    textout 11, "<'>          Invert text color"
                    TextOutLine dial_width,sh-dial_height*1.5-1 , fps+"  FPS" , False,True , 64,160,88 , 0,0,0
                End If
            End If
            SetBuffer BackBuffer()
        End If
        DrawImage textimage , 0,0
    End If
    
    ; lower-right speed indicator
    If Not first_transition
        dial_angle = dial_angle + 8
        If dial_angle >= 360
            dial_angle = dial_angle - 360
        End If
        DrawDial sw-dial_width*2-1,sh-dial_height*2-1 , dial_width,dial_height , dial_angle
    End If
    
    Flip vsync
    Cls
    If vsync
        ; slow down more (lower cpu usage)
        Delay 5
    Else
        ; fast, for debug
        Delay 1
    End If
    
Until KeyHit(1)

BASSMOD_MusicStop
BASSMOD_Free
End

Const infinity = 2147483647

Function music_order()
    Return BASSMOD_MusicGetPosition() And $0000FFFF
End Function
Function music_row()
    Return (BASSMOD_MusicGetPosition() And $FFFF0000) Shr 16
End Function

Function textout( y_mul , t$ )
    If detail_text
        TextOutLine tx,ty+y_mul*sy , t , False,False , r1,g1,b1 ,r2,g2,b2
    Else
        Color r1,g1,b1
        Text tx,ty+y_mul*sy , t
    End If
End Function
Function textoutR( y_mul , t$ )
    If detail_text
        TextOutLine sw-tx,ty+y_mul*sy , t , 2,False , r1,g1,b1 ,r2,g2,b2
    Else
        Color r1,g1,b1
        Text sw-tx-StringWidth(t),ty+y_mul*sy , t
    End If
End Function

Function DrawDial( x,y , w,h , angle , gap_angle=45,marker_diameter=5 , r#=0.2,g#=0.6,b#=0.3 )
    Local end_angle = angle
    Local start_angle = end_angle - 180
    Local draw_angle = start_angle
    Local angle_dist = end_angle - start_angle
    Local bright_high = 255
    Local bright_low = 128
    Repeat
        Local brightness = ( draw_angle - start_angle ) * (bright_high-bright_low) / angle_dist + bright_low
        Color r*brightness,g*brightness,b*brightness
        Local draw_x = x + VectorX( w , draw_angle )
        Local draw_y = y + VectorY( h , draw_angle )
        Oval draw_x,draw_y , marker_diameter,marker_diameter , True
        draw_angle = draw_angle + gap_angle
    Until draw_angle > end_angle
End Function

;-------------------------------------------------------------------------------
; image transition effect
; - assumes that the images fit the screen dimensions
Global trans_init  ; system has been initialized (automatic)
Global trans_timer
Global trans_active  ; transition is underway
Global trans_newimg  ; new image to transition to   [reference only]
Global trans_oldimg  ; old image to transition from (optional)   [reference only]
Global trans_sw,trans_sh  ; screen dimensions
Function StartTransition( newimg , oldimg=0 , duration=1000 )
    If Not trans_active
        If Not trans_init
            trans_timer = NewTimer( duration )
            trans_init = True
            trans_sw = GraphicsWidth()
            trans_sh = GraphicsHeight()
        Else
            ResetTimer trans_timer , duration
        End If
        trans_newimg = newimg
        trans_oldimg = oldimg
        trans_active = True
    End If
End Function
Function StopTransition()
    If trans_init And trans_active
        ElapseTimer trans_timer
    End If
End Function
Function UpdateTransition()
    If trans_init And trans_active
        If TimerDone( trans_timer )
            trans_active = False
            DrawBlock trans_newimg , 0,0
            Return True
        End If
        ; use the old image as background (if available)
        If trans_oldimg
            DrawBlock trans_oldimg , 0,0
        End If
        ; paint the new image in transition style on top
        Local delta# = TimerDelta( trans_timer ) * trans_sw
        Local xL = delta - trans_sw  ; left pos
        Local xR = -delta + trans_sw  ; right pos
        Local y
        ; even lines coming from left side
        For y = 0 To trans_sh-1 Step 2
            DrawBlockRect trans_newimg , xL,y , 0,y,trans_sw,1
        Next
        ; odd lines coming from right side
        For y = 1 To trans_sh-1 Step 2
            DrawBlockRect trans_newimg , xR,y , 0,y,trans_sw,1
        Next
    End If
End Function
Function TransitionActive()
    If trans_init
        Return trans_active
    End If
End Function

;-------------------------------------------------------------------------------
; contact: Teasy <TheChange@yahoo.com>

;~IDEal Editor Parameters:
;~C#Blitz3D1105